@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,24 +10,12 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/time.h>
-#include <sys/file.h>
-#include <netinet/in.h>
-#include <sys/socket.h>
-#include <netdb.h>
-#endif
-#include <stdlib.h>
-#include <string.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <lwp.h>
 #include <lock.h>
-#include <errno.h>
 #include <afs/tcdata.h>
 #include <afs/bubasics.h>
 #include <afs/budb_client.h>
@@ -36,6 +24,7 @@
 #include <afs/ktime.h>
 #include <afs/vlserver.h>
 #include <afs/volser.h>
+#include <afs/volser_prototypes.h>
 #include <afs/volint.h>
 #include <afs/cellconfig.h>
 #include <afs/bucoord_prototypes.h>
@@ -45,7 +34,6 @@
 #include "butc_xbsa.h"
 #include "afs/butx.h"
 
-
 /* GLOBAL CONFIGURATION PARAMETERS */
 extern int dump_namecheck;
 extern int queryoperator;
@@ -56,8 +44,6 @@ extern struct ubik_client *cstruct;
 dlqlinkT savedEntries;
 dlqlinkT entries_to_flush;
 
-extern struct rx_connection *UV_Bind();
-
 extern afs_int32 groupId;
 extern afs_int32 BufferSize;
 extern afs_int32 statusSize;
@@ -88,7 +74,7 @@ afs_int32 tapeblocks;		/* Number of 16K tape datablocks in buffer (!CONF_XBSA) *
  * 6) Ensure bucoord status calls work
  *
  * notes
- * pass 3: 
+ * pass 3:
  *	keep token timeout. If no user reponse (idle time > some period)
  *	and tokens about to time out, terminate dump. This provides at
  *	least something usable.
@@ -100,15 +86,6 @@ afs_int32 tapeblocks;		/* Number of 16K tape datablocks in buffer (!CONF_XBSA) *
    else \
      sprintf(dumpname, "%s (DumpId %u)", name, dbDumpId);
 
-#if defined(AFS_NT40_ENV) || (defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN60_ENV)) || defined(AFS_SUN4_ENV)
-int
-localtime_r(time_t * t, struct tm *tm)
-{
-    memcpy(tm, localtime(t), sizeof(struct tm));
-    return 0;
-}
-#endif
-
 struct dumpRock {
     /* status only */
     int tapeSeq;
@@ -167,11 +144,9 @@ calcExpirationDate(afs_int32 expType, afs_int32 expDate, afs_int32 createTime)
 	 */
 	Int32To_ktimeRelDate(expDate, &kd);
 	return (Add_RelDate_to_Time(&kd, createTime));
-	break;
 
     case BC_ABS_EXPDATE:
 	return (expDate);
-	break;
 
     case BC_NO_EXPDATE:
     default:
@@ -179,11 +154,11 @@ calcExpirationDate(afs_int32 expType, afs_int32 expDate, afs_int32 createTime)
     }
 }
 
-afs_int32 curr_bserver = 0;
+afs_uint32 curr_bserver = 0;
 struct rx_connection *curr_fromconn = (struct rx_connection *)0;
 
 struct rx_connection *
-Bind(afs_int32 server)
+Bind(afs_uint32 server)
 {
     if (curr_fromconn) {
 	if (curr_bserver == server)	/* Keep connection if have it */
@@ -206,10 +181,10 @@ Bind(afs_int32 server)
 /* notes
  * 1) save the chunksize or otherwise ensure tape space remaining is
  *	check frequently enough
- * 2) This is called once. For partial dumps, need to 
+ * 2) This is called once. For partial dumps, need to
  *	ensure that the tape device is left in the correct state for
  *	further dumps.
- * 
+ *
  */
 #define BIGCHUNK 102400
 
@@ -284,7 +259,7 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
     buffer = bufferBlock + BUTM_HDRSIZE;
 
     /* Dump one volume fragment at a time until we dump the full volume.
-     * A volume with more than 1 fragment means the volume will 'span' 
+     * A volume with more than 1 fragment means the volume will 'span'
      * 2 or more tapes.
      */
     for (fragmentNumber = 1; !endofvolume; fragmentNumber++) {	/*frag */
@@ -301,7 +276,7 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 	hostVolumeHeader.contd = ((fragmentNumber == 1) ? 0 : TC_VOLCONTD);
 	volumeHeader_hton(&hostVolumeHeader, (struct volumeHeader *)buffer);
 
-	rc = butm_WriteFileData(tapeInfoPtr, (struct volumeHeader *)buffer, 1,
+	rc = butm_WriteFileData(tapeInfoPtr, buffer, 1,
 				sizeof(hostVolumeHeader));
 	if (rc) {
 	    ErrorLog(1, taskId, rc, tapeInfoPtr->error,
@@ -338,11 +313,11 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 	    /* Determine how much data to read in upcoming RX_Read() call */
 	    toread = dataSize;
 	    /* Check if we are close to the EOT. There should at least be some
-	     * data on the tape before it is switched. HACK: we have to split a 
-	     * volume across tapes because the volume trailer says the dump 
-	     * continues on the next tape (and not the filemark). This could 
-	     * result in a volume starting on one tape (no volume data dumped) and 
-	     * continued on the next tape. It'll work, just requires restore to 
+	     * data on the tape before it is switched. HACK: we have to split a
+	     * volume across tapes because the volume trailer says the dump
+	     * continues on the next tape (and not the filemark). This could
+	     * result in a volume starting on one tape (no volume data dumped) and
+	     * continued on the next tape. It'll work, just requires restore to
 	     * switch tapes. This allows many small volumes (<16K) to be dumped.
 	     */
 	    kRemaining = butm_remainingKSpace(tapeInfoPtr);
@@ -393,7 +368,7 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 		hostVolumeHeader.contd = (endofvolume ? 0 : TC_VOLCONTD);
 		hostVolumeHeader.magic = TC_VOLENDMAGIC;
 		hostVolumeHeader.endTime = (endofvolume ? time(0) : 0);
-		volumeHeader_hton(&hostVolumeHeader, &buffer[bytesread]);
+		volumeHeader_hton(&hostVolumeHeader, (struct volumeHeader *)&buffer[bytesread]);
 		bytesread += sizeof(hostVolumeHeader);
 	    }
 
@@ -482,7 +457,7 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
     dparamsPtr->curVolumeStatus = DUMP_SUCCESS;
 
   error_exit:
-    /* 
+    /*
      * If we hit the end, see if this is the first volume on the tape or not.
      * Also, mark the tape as finished if the tape contains other dumps.
      */
@@ -505,7 +480,7 @@ dumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 	}
     }
 
-    /* 
+    /*
      * This is used when an error occurs part way into a volume dump. Clean
      * the tape state by writing an FileEnd mark. Forgo this action if we hit
      * the end of tape.
@@ -712,7 +687,7 @@ xbsaDumpVolume(struct tc_dumpDesc * curDump, struct dumpRock * dparamsPtr)
 	    bytesread += sizeof(hostVolumeHeader);
 
 	    /* End the dump and transaction with the volserver. We end it now, before
-	     * we make the XBSA call because if XBSA blocks, we could time out on the 
+	     * we make the XBSA call because if XBSA blocks, we could time out on the
 	     * volserver (After last read, the transaction with the volserver is idle).
 	     */
 	    rc = rx_EndCall(fromcall, 0);
@@ -939,7 +914,6 @@ dumpPass(struct dumpRock * dparamsPtr, int passNumber)
 			 curDump->name, curDump->vid);
 		curDump->hostAddr = 0;
 		continue;
-		break;
 	    }
 
 	    if (e >= vldbEntry.nServers) {
@@ -978,8 +952,8 @@ dumpPass(struct dumpRock * dparamsPtr, int passNumber)
 	if (checkAbortByTaskId(taskId))
 	    ERROR_EXIT(TC_ABORTEDBYREQUEST);
 
-	/* Establish connection to volume - UV_ routine expects 
-	 * host address in network order 
+	/* Establish connection to volume - UV_ routine expects
+	 * host address in network order
 	 */
 	if (CONF_XBSA) {
 	    dvcode = xbsaDumpVolume(curDump, dparamsPtr);
@@ -1031,7 +1005,6 @@ dumpPass(struct dumpRock * dparamsPtr, int passNumber)
 		case 'r':	/* retry */
 		    dparamsPtr->curVolume--;	/* redump this volume */
 		    continue;
-		    break;
 		case 'o':	/* omit */
 		    ErrorLog(1, taskId, 0, 0, "Volume %s (%u) omitted\n",
 			     curDump->name, curDump->vid);
@@ -1063,7 +1036,6 @@ dumpPass(struct dumpRock * dparamsPtr, int passNumber)
 
 	    dparamsPtr->curVolume--;	/* redump this volume */
 	    continue;
-	    break;
 
 	case DUMP_NORETRYEOT:
 	    ErrorLog(1, taskId, 0, 0,
@@ -1156,6 +1128,7 @@ Dumper(void *param)
     extern struct deviceSyncNode *deviceLatch;
     extern struct tapeConfig globalTapeConfig;
 
+    afs_pthread_setname_self("dumper");
     taskId = nodePtr->taskID;	/* Get task Id */
     setStatus(taskId, DRIVE_WAIT);
     EnterDeviceQueue(deviceLatch);
@@ -1272,7 +1245,7 @@ Dumper(void *param)
 	    break;
     }
 
-    /* 
+    /*
      * Log the error but ignore it since the dump is effectively done.
      * Scantape may assume another volume and ask for next tape.
      */
@@ -1392,7 +1365,7 @@ Dumper(void *param)
  *	volume parameters describe the volume that failed
  * entry:
  *	volumeName - name of volume
- *	volumeId - volume id 
+ *	volumeId - volume id
  *	taskId - for job contrl
  * fn return:
  *	character typed by user, one of r, o or a
@@ -1422,11 +1395,7 @@ retryPrompt(char *volumeName, afs_int32 volumeId, afs_uint32 taskId)
 
 	start = time(0);
 	while (1) {
-#ifdef AFS_PTHREAD_ENV
-	    code = GetResponseKey(5, &ch);	/* ch stores key pressed */
-#else
 	    code = LWP_GetResponseKey(5, &ch);	/* ch stores key pressed */
-#endif
 	    if (code == 1)
 		break;		/* input is available */
 
@@ -1754,7 +1723,7 @@ getDumpTape(struct dumpRock *dparamsPtr, int interactiveFlag,
 
 	/*
 	 * Now have the right tape. Create a new label for the tape
-	 * Appended labels have the dump's dumpId - labels at beginnings of 
+	 * Appended labels have the dump's dumpId - labels at beginnings of
 	 *     tape have the initial dump's dumpId.
 	 * Appended labels do not increment the useCount.
 	 * Labels at beginnings of tape use the most future expiration of the dump set.
@@ -1884,7 +1853,7 @@ makeVolumeHeader(struct volumeHeader *vhptr, struct dumpRock *dparamsPtr,
     strcpy(vhptr->dumpSetName, nodePtr->dumpSetName);
     strcpy(vhptr->preamble, "H++NAME#");
     strcpy(vhptr->postamble, "T--NAME#");
-  
+
     return (code);
 }
 
@@ -2061,6 +2030,7 @@ DeleteDump(void *param)
     extern struct udbHandleS udbHandle;
     extern struct deviceSyncNode *deviceLatch;
 
+    afs_pthread_setname_self("deletedump");
     setStatus(taskId, DRIVE_WAIT);
     EnterDeviceQueue(deviceLatch);
     clearStatus(taskId, DRIVE_WAIT);
