@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,17 +9,11 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/sys/pioctl_nt.c,v 1.34.4.22 2009/04/06 21:47:59 jaltman Exp $");
-
 #include <afs/stds.h>
+
+#include <roken.h>
+
 #include <windows.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <errno.h>
-#include <malloc.h>
-#include <string.h>
 #include <winioctl.h>
 #include <winsock2.h>
 #define SECURITY_WIN32
@@ -39,7 +33,7 @@ RCSID
 #include <cm_dir.h>
 #include <cm_utils.h>
 #include <cm_ioctl.h>
-
+#include <smb_iocons.h>
 #include <smb.h>
 #include <pioctl_nt.h>
 #include <WINNT/afsreg.h>
@@ -47,6 +41,9 @@ RCSID
 
 #include <loadfuncs-krb5.h>
 #include <krb5.h>
+#include <..\WINNT\afsrdr\common\AFSUserDefines.h>
+#include <..\WINNT\afsrdr\common\AFSUserIoctl.h>
+#include <..\WINNT\afsrdr\common\AFSUserStructs.h>
 
 static char AFSConfigKeyName[] = AFSREG_CLT_SVC_PARAM_SUBKEY;
 
@@ -61,6 +58,7 @@ typedef struct fs_ioctlRequest {
     char data[FS_IOCTLREQUEST_MAXSIZE];	/* data we're marshalling */
 } fs_ioctlRequest_t;
 
+
 static int
 CMtoUNIXerror(int cm_code)
 {
@@ -128,7 +126,7 @@ IoctlDebug(void)
     if ( !init ) {
         HKEY hk;
 
-        if (RegOpenKey (HKEY_LOCAL_MACHINE, 
+        if (RegOpenKey (HKEY_LOCAL_MACHINE,
                          TEXT("Software\\OpenAFS\\Client"), &hk) == 0)
         {
             DWORD dwSize = sizeof(BOOL);
@@ -143,6 +141,111 @@ IoctlDebug(void)
     return debug;
 }
 
+static BOOL
+RDR_Ready(void)
+{
+    HANDLE hDevHandle = NULL;
+    DWORD bytesReturned;
+    AFSDriverStatusRespCB *respBuffer = NULL;
+    DWORD rc = 0;
+    BOOL ready = FALSE;
+
+    hDevHandle = CreateFileW( AFS_SYMLINK_W,
+                             GENERIC_READ | GENERIC_WRITE,
+                             FILE_SHARE_READ | FILE_SHARE_WRITE,
+                             NULL,
+                             OPEN_EXISTING,
+                             0,
+                             NULL);
+    if( hDevHandle == INVALID_HANDLE_VALUE)
+    {
+        DWORD gle = GetLastError();
+
+        if (gle && IoctlDebug() ) {
+            char buf[4096];
+            int saveerrno;
+
+            saveerrno = errno;
+            if ( FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
+                               NULL,
+                               gle,
+                               MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US),
+                               buf,
+                               4096,
+                               (va_list *) NULL
+                               ) )
+            {
+                fprintf(stderr,"RDR_Ready CreateFile(%S) failed: 0x%X\r\n\t[%s]\r\n",
+                        AFS_SYMLINK_W,gle,buf);
+            }
+            errno = saveerrno;
+        }
+        return FALSE;
+    }
+
+    //
+    // Allocate a response buffer.
+    //
+    respBuffer = (AFSDriverStatusRespCB *)malloc( sizeof( AFSDriverStatusRespCB));
+    if( respBuffer)
+    {
+
+	memset( respBuffer, '\0', sizeof( AFSDriverStatusRespCB));
+
+        if( !DeviceIoControl( hDevHandle,
+                              IOCTL_AFS_STATUS_REQUEST,
+                              NULL,
+                              0,
+                              (void *)respBuffer,
+                              sizeof( AFSDriverStatusRespCB),
+                              &bytesReturned,
+                              NULL))
+        {
+            //
+            // Error condition back from driver
+            //
+            DWORD gle = GetLastError();
+
+            if (gle && IoctlDebug() ) {
+                char buf[4096];
+                int saveerrno;
+
+                saveerrno = errno;
+                if ( FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
+                                    NULL,
+                                    gle,
+                                    MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US),
+                                    buf,
+                                    4096,
+                                    (va_list *) NULL
+                                    ) )
+                {
+                    fprintf(stderr,"RDR_Ready CreateFile(%s) failed: 0x%X\r\n\t[%s]\r\n",
+                             AFS_SYMLINK,gle,buf);
+                }
+                errno = saveerrno;
+            }
+            rc = -1;
+            goto cleanup;
+        }
+
+        if (bytesReturned == sizeof(AFSDriverStatusRespCB))
+        {
+            ready = ( respBuffer->Status == AFS_DRIVER_STATUS_READY );
+        }
+    } else
+        rc = ENOMEM;
+
+  cleanup:
+    if (respBuffer)
+        free( respBuffer);
+
+    if (hDevHandle != INVALID_HANDLE_VALUE)
+        CloseHandle(hDevHandle);
+
+    return ready;
+}
+
 static BOOL
 DisableServiceManagerCheck(void)
 {
@@ -152,7 +255,7 @@ DisableServiceManagerCheck(void)
     if ( !init ) {
         HKEY hk;
 
-        if (RegOpenKey (HKEY_LOCAL_MACHINE, 
+        if (RegOpenKey (HKEY_LOCAL_MACHINE,
                          TEXT("Software\\OpenAFS\\Client"), &hk) == 0)
         {
             DWORD dwSize = sizeof(BOOL);
@@ -167,61 +270,61 @@ DisableServiceManagerCheck(void)
     return smcheck;
 }
 
-static DWORD 
+static DWORD
 GetServiceStatus(
-    LPSTR lpszMachineName, 
+    LPSTR lpszMachineName,
     LPSTR lpszServiceName,
-    DWORD *lpdwCurrentState) 
-{ 
-    DWORD           hr               = NOERROR; 
-    SC_HANDLE       schSCManager     = NULL; 
-    SC_HANDLE       schService       = NULL; 
-    DWORD           fdwDesiredAccess = 0; 
-    SERVICE_STATUS  ssServiceStatus  = {0}; 
-    BOOL            fRet             = FALSE; 
+    DWORD *lpdwCurrentState)
+{
+    DWORD           hr               = NOERROR;
+    SC_HANDLE       schSCManager     = NULL;
+    SC_HANDLE       schService       = NULL;
+    DWORD           fdwDesiredAccess = 0;
+    SERVICE_STATUS  ssServiceStatus  = {0};
+    BOOL            fRet             = FALSE;
 
-    *lpdwCurrentState = 0; 
- 
-    fdwDesiredAccess = GENERIC_READ; 
- 
-    schSCManager = OpenSCManager(lpszMachineName,  
+    *lpdwCurrentState = 0;
+
+    fdwDesiredAccess = GENERIC_READ;
+
+    schSCManager = OpenSCManager(lpszMachineName,
                                  NULL,
-                                 fdwDesiredAccess); 
- 
-    if(schSCManager == NULL) 
-    { 
+                                 fdwDesiredAccess);
+
+    if(schSCManager == NULL)
+    {
         hr = GetLastError();
-        goto cleanup; 
-    } 
- 
+        goto cleanup;
+    }
+
     schService = OpenService(schSCManager,
                              lpszServiceName,
-                             fdwDesiredAccess); 
- 
-    if(schService == NULL) 
-    { 
+                             fdwDesiredAccess);
+
+    if(schService == NULL)
+    {
         hr = GetLastError();
-        goto cleanup; 
-    } 
- 
+        goto cleanup;
+    }
+
     fRet = QueryServiceStatus(schService,
-                              &ssServiceStatus); 
- 
-    if(fRet == FALSE) 
-    { 
-        hr = GetLastError(); 
-        goto cleanup; 
-    } 
- 
-    *lpdwCurrentState = ssServiceStatus.dwCurrentState; 
- 
-cleanup: 
- 
-    CloseServiceHandle(schService); 
-    CloseServiceHandle(schSCManager); 
- 
-    return(hr); 
-} 
+                              &ssServiceStatus);
+
+    if(fRet == FALSE)
+    {
+        hr = GetLastError();
+        goto cleanup;
+    }
+
+    *lpdwCurrentState = ssServiceStatus.dwCurrentState;
+
+cleanup:
+
+    CloseServiceHandle(schService);
+    CloseServiceHandle(schSCManager);
+
+    return(hr);
+}
 
 // krb5 functions
 DECL_FUNC_PTR(krb5_cc_default_name);
@@ -427,7 +530,7 @@ DriveSubstitution(char *drivestr, char *subststr, size_t substlen)
                 subststr[2] = '\0';
                 return TRUE;
             }
-        } else 
+        } else
         if ( device[0] == '\\' &&
              device[1] == '?' &&
              device[2] == '?' &&
@@ -447,7 +550,7 @@ DriveSubstitution(char *drivestr, char *subststr, size_t substlen)
     return FALSE;
 }
 
-// 
+//
 // drivestr - is "<drive-letter>:"
 //
 static BOOL
@@ -461,6 +564,7 @@ DriveIsMappedToAFS(char *drivestr, char *NetbiosName)
     DWORD i;
     BOOL  bIsAFS = FALSE;
     char  subststr[MAX_PATH];
+    char  device[MAX_PATH];
 
     //
     // Handle drive letter substitution created with "SUBST <drive> <path>".
@@ -470,7 +574,7 @@ DriveIsMappedToAFS(char *drivestr, char *NetbiosName)
     if ( DriveSubstitution(drivestr, subststr, MAX_PATH) )
     {
         if (subststr[0] == '\\' &&
-            subststr[1] == '\\') 
+            subststr[1] == '\\')
         {
             if (_strnicmp( &subststr[2], NetbiosName, strlen(NetbiosName)) == 0)
                 return TRUE;
@@ -480,6 +584,14 @@ DriveIsMappedToAFS(char *drivestr, char *NetbiosName)
         drivestr = subststr;
     }
 
+    //
+    // Check for \Device\AFSRedirector
+    //
+    if (QueryDosDevice(drivestr, device, MAX_PATH) &&
+        _strnicmp( device, "\\Device\\AFSRedirector", strlen("\\Device\\AFSRedirector")) == 0) {
+        return TRUE;
+    }
+
     //
     // Call the WNetOpenEnum function to begin the enumeration.
     //
@@ -537,7 +649,7 @@ DriveIsMappedToAFS(char *drivestr, char *NetbiosName)
             break;
     }
     while (dwResultEnum != ERROR_NO_MORE_ITEMS);
-    
+
     //
     // Call the GlobalFree function to free the memory.
     //
@@ -559,8 +671,8 @@ DriveIsGlobalAutoMapped(char *drivestr)
     char szSubMount[260];
     DWORD dwType;
 
-    dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
-                            AFSREG_CLT_SVC_PARAM_SUBKEY "\\GlobalAutoMapper", 
+    dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+                            AFSREG_CLT_SVC_PARAM_SUBKEY "\\GlobalAutoMapper",
                             0, KEY_QUERY_VALUE, &hKey);
     if (dwResult != ERROR_SUCCESS)
         return FALSE;
@@ -579,13 +691,13 @@ DriveIsGlobalAutoMapped(char *drivestr)
 static long
 GetIoctlHandle(char *fileNamep, HANDLE * handlep)
 {
+    HKEY hk;
     char *drivep = NULL;
-    char netbiosName[MAX_NB_NAME_LENGTH];
+    char netbiosName[MAX_NB_NAME_LENGTH]="AFS";
     DWORD CurrentState = 0;
     char  HostName[64] = "";
     char tbuffer[MAX_PATH]="";
     HANDLE fh;
-    HKEY hk;
     char szUser[128] = "";
     char szClient[MAX_PATH] = "";
     char szPath[MAX_PATH] = "";
@@ -593,19 +705,74 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
     DWORD res;
     DWORD ioctlDebug = IoctlDebug();
     DWORD gle;
+    DWORD dwAttrib;
     DWORD dwSize = sizeof(szUser);
+    BOOL  usingRDR = FALSE;
     int saveerrno;
     UINT driveType;
+    int sharingViolation;
 
     memset(HostName, '\0', sizeof(HostName));
     gethostname(HostName, sizeof(HostName));
     if (!DisableServiceManagerCheck() &&
         GetServiceStatus(HostName, TEXT("TransarcAFSDaemon"), &CurrentState) == NOERROR &&
-	CurrentState != SERVICE_RUNNING)
+        CurrentState != SERVICE_RUNNING)
+    {
+        if ( ioctlDebug ) {
+            saveerrno = errno;
+            fprintf(stderr, "pioctl GetServiceStatus(%s) == %d\r\n",
+                    HostName, CurrentState);
+            errno = saveerrno;
+        }
 	return -1;
+    }
 
-    // Populate the Netbios Name
-    lana_GetNetbiosName(netbiosName,LANA_NETBIOS_NAME_FULL);
+    if (RDR_Ready()) {
+        usingRDR = TRUE;
+
+        if ( ioctlDebug ) {
+            saveerrno = errno;
+            fprintf(stderr, "pioctl Redirector is ready\r\n");
+            errno = saveerrno;
+        }
+
+        if (RegOpenKey (HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY, &hk) == 0)
+        {
+            DWORD dwSize = sizeof(netbiosName);
+            DWORD dwType = REG_SZ;
+            RegQueryValueExA (hk, "NetbiosName", NULL, &dwType, (PBYTE)netbiosName, &dwSize);
+            RegCloseKey (hk);
+
+            if ( ioctlDebug ) {
+                saveerrno = errno;
+                fprintf(stderr, "pioctl NetbiosName = \"%s\"\r\n", netbiosName);
+                errno = saveerrno;
+            }
+        } else {
+            if ( ioctlDebug ) {
+                saveerrno = errno;
+                gle = GetLastError();
+                fprintf(stderr, "pioctl Unable to open \"HKLM\\%s\" using NetbiosName = \"AFS\" GLE=0x%x\r\n",
+                        HostName, CurrentState, gle);
+                errno = saveerrno;
+            }
+        }
+    } else {
+        if ( ioctlDebug ) {
+            saveerrno = errno;
+            fprintf(stderr, "pioctl Redirector is not ready\r\n");
+            errno = saveerrno;
+        }
+
+        if (!GetEnvironmentVariable("AFS_PIOCTL_SERVER", netbiosName, sizeof(netbiosName)))
+            lana_GetNetbiosName(netbiosName,LANA_NETBIOS_NAME_FULL);
+
+        if ( ioctlDebug ) {
+            saveerrno = errno;
+            fprintf(stderr, "pioctl NetbiosName = \"%s\"\r\n", netbiosName);
+            errno = saveerrno;
+        }
+    }
 
     if (fileNamep) {
         drivep = strchr(fileNamep, ':');
@@ -621,16 +788,16 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                 if (DriveIsMappedToAFS(tbuffer, netbiosName) ||
                     DriveIsGlobalAutoMapped(tbuffer))
                     strcpy(&tbuffer[2], SMB_IOCTL_FILENAME);
-                else 
+                else
                     return -1;
                 break;
             default:
                 if (DriveIsGlobalAutoMapped(tbuffer))
                     strcpy(&tbuffer[2], SMB_IOCTL_FILENAME);
-                else 
+                else
                     return -1;
             }
-        } else if (fileNamep[0] == fileNamep[1] && 
+        } else if (fileNamep[0] == fileNamep[1] &&
 		   (fileNamep[0] == '\\' || fileNamep[0] == '/'))
         {
             int count = 0, i = 0;
@@ -642,10 +809,10 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                     count++;
 		i++;
             }
-            if (fileNamep[i] == 0)
+            if (fileNamep[i] == 0 || (fileNamep[i-1] != '\\' && fileNamep[i-1] != '/'))
                 tbuffer[i++] = '\\';
             tbuffer[i] = 0;
-            strcat(tbuffer, SMB_IOCTL_FILENAME);
+            strcat(tbuffer, SMB_IOCTL_FILENAME_NOSLASH);
         } else {
             char curdir[MAX_PATH]="";
 
@@ -662,17 +829,17 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                     if (DriveIsMappedToAFS(tbuffer, netbiosName) ||
                         DriveIsGlobalAutoMapped(tbuffer))
                         strcpy(&tbuffer[2], SMB_IOCTL_FILENAME);
-                    else 
+                    else
                         return -1;
                     break;
                 default:
                     if (DriveIsGlobalAutoMapped(tbuffer))
                         strcpy(&tbuffer[2], SMB_IOCTL_FILENAME);
-                    else 
+                    else
                         return -1;
                 }
             } else if (curdir[0] == curdir[1] &&
-                       (curdir[0] == '\\' || curdir[0] == '/')) 
+                       (curdir[0] == '\\' || curdir[0] == '/'))
             {
                 int count = 0, i = 0;
 
@@ -683,7 +850,7 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
 			count++;
 		    i++;
                 }
-                if (tbuffer[i] == 0)
+                if (curdir[i] == 0 || (curdir[i-1] != '\\' && curdir[i-1] != '/'))
                     tbuffer[i++] = '\\';
                 tbuffer[i] = 0;
                 strcat(tbuffer, SMB_IOCTL_FILENAME_NOSLASH);
@@ -695,21 +862,25 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
         sprintf(tbuffer,"\\\\%s\\all%s",netbiosName,SMB_IOCTL_FILENAME);
     }
 
+    if ( ioctlDebug ) {
+        saveerrno = errno;
+        fprintf(stderr, "pioctl filename = \"%s\"\r\n", tbuffer);
+        errno = saveerrno;
+    }
+
     fflush(stdout);
-    /* now open the file */
-    fh = CreateFile(tbuffer, GENERIC_READ | GENERIC_WRITE,
-		    FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
-		    FILE_FLAG_WRITE_THROUGH, NULL);
 
-	fflush(stdout);
-
-    if (fh == INVALID_HANDLE_VALUE) {
+    /*
+     * Try to find the correct path and authentication
+     */
+    dwAttrib = GetFileAttributes(tbuffer);
+    if (dwAttrib == INVALID_FILE_ATTRIBUTES) {
         int  gonext = 0;
 
         gle = GetLastError();
         if (gle && ioctlDebug ) {
             char buf[4096];
-            
+
             saveerrno = errno;
             if ( FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                                NULL,
@@ -720,15 +891,21 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                                (va_list *) NULL
                                ) )
             {
-                fprintf(stderr,"pioctl CreateFile(%s) failed: 0x%X\r\n\t[%s]\r\n",
+                fprintf(stderr,"pioctl GetFileAttributes(%s) failed: 0x%X\r\n\t[%s]\r\n",
                         tbuffer,gle,buf);
             }
             errno = saveerrno;
+            SetLastError(gle);
         }
 
-        lana_GetNetbiosName(szClient, LANA_NETBIOS_NAME_FULL);
+        /* with the redirector interface, fail immediately.  there is nothing to retry */
+        if (usingRDR)
+            return -1;
 
-        if (RegOpenKey (HKEY_CURRENT_USER, 
+        if (!GetEnvironmentVariable("AFS_PIOCTL_SERVER", szClient, sizeof(szClient)))
+            lana_GetNetbiosName(szClient, LANA_NETBIOS_NAME_FULL);
+
+        if (RegOpenKey (HKEY_CURRENT_USER,
                          TEXT("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer"), &hk) == 0)
         {
             DWORD dwType = REG_SZ;
@@ -773,11 +950,8 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
             if (gonext)
                 goto try_lsa_principal;
 
-            fh = CreateFile(tbuffer, GENERIC_READ | GENERIC_WRITE,
-                             FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
-                             FILE_FLAG_WRITE_THROUGH, NULL);
-            fflush(stdout);
-            if (fh == INVALID_HANDLE_VALUE) {
+            dwAttrib = GetFileAttributes(tbuffer);
+            if (dwAttrib == INVALID_FILE_ATTRIBUTES) {
                 gle = GetLastError();
                 if (gle && ioctlDebug ) {
                     char buf[4096];
@@ -792,17 +966,19 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                                         (va_list *) NULL
                                         ) )
                     {
-                        fprintf(stderr,"pioctl CreateFile(%s) failed: 0x%X\r\n\t[%s]\r\n",
+                        fprintf(stderr,"pioctl GetFileAttributes(%s) failed: 0x%X\r\n\t[%s]\r\n",
                                  tbuffer,gle,buf);
                     }
                     errno = saveerrno;
+                    SetLastError(gle);
                 }
             }
         }
     }
 
   try_lsa_principal:
-    if (fh == INVALID_HANDLE_VALUE) {
+    if (!usingRDR &&
+        dwAttrib == INVALID_FILE_ATTRIBUTES) {
         int  gonext = 0;
 
         dwSize = sizeof(szUser);
@@ -843,11 +1019,8 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
             if (gonext)
                 goto try_sam_compat;
 
-            fh = CreateFile(tbuffer, GENERIC_READ | GENERIC_WRITE,
-                             FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
-                             FILE_FLAG_WRITE_THROUGH, NULL);
-            fflush(stdout);
-            if (fh == INVALID_HANDLE_VALUE) {
+            dwAttrib = GetFileAttributes(tbuffer);
+            if (dwAttrib == INVALID_FILE_ATTRIBUTES) {
                 gle = GetLastError();
                 if (gle && ioctlDebug ) {
                     char buf[4096];
@@ -862,18 +1035,19 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                                         (va_list *) NULL
                                         ) )
                     {
-                        fprintf(stderr,"pioctl CreateFile(%s) failed: 0x%X\r\n\t[%s]\r\n",
+                        fprintf(stderr,"pioctl GetFileAttributes(%s) failed: 0x%X\r\n\t[%s]\r\n",
                                  tbuffer,gle,buf);
                     }
                     errno = saveerrno;
-
+                    SetLastError(gle);
                 }
             }
         }
     }
 
   try_sam_compat:
-    if ( fh == INVALID_HANDLE_VALUE ) {
+    if (!usingRDR &&
+        dwAttrib == INVALID_FILE_ATTRIBUTES) {
         dwSize = sizeof(szUser);
         if (GetUserNameEx(NameSamCompatible, szUser, &dwSize)) {
             if ( ioctlDebug ) {
@@ -908,11 +1082,8 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                 return -1;
             }
 
-            fh = CreateFile(tbuffer, GENERIC_READ | GENERIC_WRITE,
-                             FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
-                             FILE_FLAG_WRITE_THROUGH, NULL);
-            fflush(stdout);
-            if (fh == INVALID_HANDLE_VALUE) {
+            dwAttrib = GetFileAttributes(tbuffer);
+            if (dwAttrib == INVALID_FILE_ATTRIBUTES) {
                 gle = GetLastError();
                 if (gle && ioctlDebug ) {
                     char buf[4096];
@@ -927,7 +1098,7 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
                                         (va_list *) NULL
                                         ) )
                     {
-                        fprintf(stderr,"pioctl CreateFile(%s) failed: 0x%X\r\n\t[%s]\r\n",
+                        fprintf(stderr,"pioctl GetFileAttributes(%s) failed: 0x%X\r\n\t[%s]\r\n",
                                  tbuffer,gle,buf);
                     }
                     errno = saveerrno;
@@ -940,6 +1111,29 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
         }
     }
 
+    if ( dwAttrib != (FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM)) {
+        fprintf(stderr, "GetFileAttributes(%s) returned: 0x%08X\r\n",
+                tbuffer, dwAttrib);
+        return -1;
+    }
+
+    /* tbuffer now contains the correct path; now open the file */
+    sharingViolation = 0;
+    do {
+        if (sharingViolation)
+            Sleep(100);
+        fh = CreateFile(tbuffer, FILE_READ_DATA | FILE_WRITE_DATA,
+                        FILE_SHARE_READ, NULL, OPEN_EXISTING,
+                        FILE_FLAG_WRITE_THROUGH, NULL);
+        sharingViolation++;
+    } while (fh == INVALID_HANDLE_VALUE &&
+             GetLastError() == ERROR_SHARING_VIOLATION &&
+             sharingViolation < 100);
+    fflush(stdout);
+
+    if (fh == INVALID_HANDLE_VALUE)
+        return -1;
+
     /* return fh and success code */
     *handlep = fh;
     return 0;
@@ -1091,7 +1285,7 @@ fs_GetFullPath(char *pathp, char *outPathp, long outSize)
 	pathHasDrive = 0;
     }
 
-    if ( firstp[0] == '\\' && firstp[1] == '\\' || 
+    if ( firstp[0] == '\\' && firstp[1] == '\\' ||
 	 firstp[0] == '/' && firstp[1] == '/') {
         /* UNC path - strip off the server and sharename */
         int i, count;
@@ -1171,7 +1365,7 @@ fs_GetFullPath(char *pathp, char *outPathp, long outSize)
     /* if there is a non-null name after the drive, append it */
     if (*firstp != 0) {
         int len = (int)strlen(outPathp);
-        if (outPathp[len-1] != '\\' && outPathp[len-1] != '/') 
+        if (outPathp[len-1] != '\\' && outPathp[len-1] != '/')
             strcat(outPathp, "\\");
         strcat(outPathp, firstp);
     }
@@ -1188,15 +1382,104 @@ fs_GetFullPath(char *pathp, char *outPathp, long outSize)
     return 0;
 }
 
-static long
-pioctl_int(char *pathp, long opcode, struct ViceIoctl *blobp, int follow, int is_utf8)
+static int
+pioctl_int(char *pathp, afs_int32 opcode, struct ViceIoctl *blobp, afs_int32 follow, afs_int32 is_utf8)
 {
     fs_ioctlRequest_t preq;
     long code;
     long temp;
     char fullPath[1000];
+    char altPath[1024];
     HANDLE reqHandle;
     int save;
+    int i,j,count,all;
+
+    /*
+     * The pioctl operations for creating a mount point and a symlink are broken.
+     * Instead of 'pathp' referring to the directory object in which the symlink
+     * or mount point within which the new object is to be created, 'pathp' refers
+     * to the object itself.  This results in a problem when the object being created
+     * is located within the Freelance root.afs volume.  \\afs\foo will not be a
+     * valid share name since the 'foo' object does not yet exist.  Therefore,
+     * \\afs\foo\_._.afs_ioctl_._ cannot be opened.  Instead in these two cases
+     * we must force the use of the \\afs\all\foo form of the path.
+     *
+     * We cannot use this form in all cases because of smb submounts which are
+     * not located within the Freelance local root.
+     */
+    switch ( opcode ) {
+    case VIOC_AFS_CREATE_MT_PT:
+    case VIOC_SYMLINK:
+        if (pathp &&
+             (pathp[0] == '\\' && pathp[1] == '\\' ||
+              pathp[0] == '/' && pathp[1] == '/')) {
+            for (all = count = j = 0; pathp[j]; j++) {
+                if (pathp[j] == '\\' || pathp[j] == '/')
+                    count++;
+
+                /* Test to see if the second component is 'all' */
+                if (count == 3) {
+                    all = 1;
+                    for (i=0; pathp[i+j]; i++) {
+                        switch(i) {
+                        case 0:
+                            if (pathp[i+j] != 'a' &&
+                                pathp[i+j] != 'A') {
+                                all = 0;
+                                goto notall;
+                            }
+                            break;
+                        case 1:
+                        case 2:
+                            if (pathp[i+j] != 'l' &&
+                                 pathp[i+j] != 'L') {
+                                all = 0;
+                                goto notall;
+                            }
+                            break;
+                        default:
+                            all = 0;
+                            goto notall;
+                        }
+                    }
+                    if (i != 3)
+                        all = 0;
+                }
+
+              notall:
+                if (all)
+                    break;
+            }
+
+            /*
+             * if count is three and the second component is not 'all',
+             * then we are attempting to create an object in the
+             * Freelance root.afs volume.  Substitute the path.
+             */
+
+            if (count == 3 && !all) {
+                /* Normalize the name to use \\afs\all as the root */
+                for (count = i = j = 0; pathp[j] && i < sizeof(altPath); j++) {
+                    if (pathp[j] == '\\' || pathp[j] == '/') {
+                        altPath[i++] = '\\';
+                        count++;
+
+                        if (count == 3) {
+                            altPath[i++] = 'a';
+                            altPath[i++] = 'l';
+                            altPath[i++] = 'l';
+                            altPath[i++] = '\\';
+                            count++;
+                        }
+                    } else {
+                        altPath[i++] = pathp[j];
+                    }
+                }
+                altPath[i] = '\0';
+                pathp = altPath;
+            }
+        }
+    }
 
     code = GetIoctlHandle(pathp, &reqHandle);
     if (code) {
@@ -1276,14 +1559,14 @@ pioctl_int(char *pathp, long opcode, struct ViceIoctl *blobp, int follow, int is
     return 0;
 }
 
-long
-pioctl_utf8(char * pathp, long opcode, struct ViceIoctl * blobp, int follow)
+int
+pioctl_utf8(char * pathp, afs_int32 opcode, struct ViceIoctl * blobp, afs_int32 follow)
 {
     return pioctl_int(pathp, opcode, blobp, follow, TRUE);
 }
 
-long
-pioctl(char * pathp, long opcode, struct ViceIoctl * blobp, int follow)
+int
+pioctl(char * pathp, afs_int32 opcode, struct ViceIoctl * blobp, afs_int32 follow)
 {
     return pioctl_int(pathp, opcode, blobp, follow, FALSE);
 }
